@@ -4,14 +4,35 @@ import logging
 import itertools
 import numpy as np
 from uuid import uuid4
-from sqlalchemy import and_, func, create_engine, Column, String, DateTime, ForeignKey, Boolean, Text, text, JSON, ForeignKeyConstraint
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import relationship, sessionmaker
-from sqlalchemy.sql import case, null
+
+try:
+    from sqlalchemy import (
+        and_,
+        func,
+        create_engine,
+        Column,
+        String,
+        DateTime,
+        ForeignKey,
+        Boolean,
+        Text,
+        text,
+        JSON,
+        ForeignKeyConstraint,
+    )
+    from sqlalchemy.ext.declarative import declarative_base
+    from sqlalchemy.orm import relationship, sessionmaker
+    from sqlalchemy.sql import case, null
+except (ImportError, ModuleNotFoundError) as ie:
+    from haystack.utils.import_utils import _optional_component_not_installed
+
+    _optional_component_not_installed(__name__, "sql", ie)
 
 from haystack.schema import Document, Label, Answer
 from haystack.document_stores.base import BaseDocumentStore
 
+from haystack.document_stores.filter_utils import LogicalFilterClause
+
 
 logger = logging.getLogger(__name__)
 Base = declarative_base()  # type: Any
@@ -46,9 +67,12 @@ class MetaDocumentORM(ORMBase):
 
     document_id = Column(String(100), nullable=False, index=True)
     document_index = Column(String(100), nullable=False, index=True)
-    __table_args__ = (ForeignKeyConstraint([document_id, document_index],
-                                           [DocumentORM.id, DocumentORM.index],
-                                           ondelete="CASCADE", onupdate="CASCADE"), {})  #type: ignore
+    __table_args__ = (
+        ForeignKeyConstraint(
+            [document_id, document_index], [DocumentORM.id, DocumentORM.index], ondelete="CASCADE", onupdate="CASCADE"
+        ),
+        {},
+    )  # type: ignore
 
 
 class LabelORM(ORMBase):
@@ -76,10 +100,12 @@ class MetaLabelORM(ORMBase):
 
     label_id = Column(String(100), nullable=False, index=True)
     label_index = Column(String(100), nullable=False, index=True)
-    __table_args__ = (ForeignKeyConstraint([label_id, label_index],
-                                           [LabelORM.id, LabelORM.index],
-                                           ondelete="CASCADE", onupdate="CASCADE"), {})  #type: ignore
-
+    __table_args__ = (
+        ForeignKeyConstraint(
+            [label_id, label_index], [LabelORM.id, LabelORM.index], ondelete="CASCADE", onupdate="CASCADE"
+        ),
+        {},
+    )  # type: ignore
 
 
 class SQLDocumentStore(BaseDocumentStore):
@@ -90,13 +116,13 @@ class SQLDocumentStore(BaseDocumentStore):
         label_index: str = "label",
         duplicate_documents: str = "overwrite",
         check_same_thread: bool = False,
-        isolation_level: str = None
+        isolation_level: str = None,
     ):
         """
         An SQL backed DocumentStore. Currently supports SQLite, PostgreSQL and MySQL backends.
 
         :param url: URL for SQL database as expected by SQLAlchemy. More info here: https://docs.sqlalchemy.org/en/13/core/engines.html#database-urls
-        :param index: The documents are scoped to an index attribute that can be used when writing, querying, or deleting documents. 
+        :param index: The documents are scoped to an index attribute that can be used when writing, querying, or deleting documents.
                       This parameter sets the default value for document index.
         :param label_index: The default value of index attribute for the labels.
         :param duplicate_documents: Handle duplicates document based on parameter options.
@@ -105,19 +131,23 @@ class SQLDocumentStore(BaseDocumentStore):
                                     overwrite: Update any existing documents with the same ID when adding documents.
                                     fail: an error is raised if the document ID of the document being added already
                                     exists.
-        :param check_same_thread: Set to False to mitigate multithreading issues in older SQLite versions (see https://docs.sqlalchemy.org/en/14/dialects/sqlite.html?highlight=check_same_thread#threading-pooling-behavior) 
+        :param check_same_thread: Set to False to mitigate multithreading issues in older SQLite versions (see https://docs.sqlalchemy.org/en/14/dialects/sqlite.html?highlight=check_same_thread#threading-pooling-behavior)
         :param isolation_level: see SQLAlchemy's `isolation_level` parameter for `create_engine()` (https://docs.sqlalchemy.org/en/14/core/engines.html#sqlalchemy.create_engine.params.isolation_level)
         """
 
         # save init parameters to enable export of component config as YAML
         self.set_config(
-                url=url, index=index, label_index=label_index, duplicate_documents=duplicate_documents, check_same_thread=check_same_thread
+            url=url,
+            index=index,
+            label_index=label_index,
+            duplicate_documents=duplicate_documents,
+            check_same_thread=check_same_thread,
         )
         create_engine_params = {}
         if isolation_level:
             create_engine_params["isolation_level"] = isolation_level
         if "sqlite" in url:
-            engine = create_engine(url, connect_args={'check_same_thread': check_same_thread}, **create_engine_params)
+            engine = create_engine(url, connect_args={"check_same_thread": check_same_thread}, **create_engine_params)
         else:
             engine = create_engine(url, **create_engine_params)
         Base.metadata.create_all(engine)
@@ -131,30 +161,38 @@ class SQLDocumentStore(BaseDocumentStore):
         self.use_windowed_query = True
         if "sqlite" in url:
             import sqlite3
+
             if sqlite3.sqlite_version < "3.25":
                 self.use_windowed_query = False
 
-    def get_document_by_id(self, id: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None) -> Optional[Document]:
+    def get_document_by_id(
+        self, id: str, index: Optional[str] = None, headers: Optional[Dict[str, str]] = None
+    ) -> Optional[Document]:
         """Fetch a document by specifying its text id string"""
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         documents = self.get_documents_by_id([id], index)
         document = documents[0] if documents else None
         return document
 
-    def get_documents_by_id(self, ids: List[str], index: Optional[str] = None, batch_size: int = 10_000, headers: Optional[Dict[str, str]] = None) -> List[Document]:
+    def get_documents_by_id(
+        self,
+        ids: List[str],
+        index: Optional[str] = None,
+        batch_size: int = 10_000,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> List[Document]:
         """Fetch documents by specifying a list of text id strings"""
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.index
 
         documents = []
         for i in range(0, len(ids), batch_size):
             query = self.session.query(DocumentORM).filter(
-                DocumentORM.id.in_(ids[i: i + batch_size]),
-                DocumentORM.index == index
+                DocumentORM.id.in_(ids[i : i + batch_size]), DocumentORM.index == index
             )
             for row in query.all():
                 documents.append(self._convert_sql_row_to_document(row))
@@ -168,8 +206,7 @@ class SQLDocumentStore(BaseDocumentStore):
         documents = []
         for i in range(0, len(vector_ids), batch_size):
             query = self.session.query(DocumentORM).filter(
-                DocumentORM.vector_id.in_(vector_ids[i: i + batch_size]),
-                DocumentORM.index == index
+                DocumentORM.vector_id.in_(vector_ids[i : i + batch_size]), DocumentORM.index == index
             )
             for row in query.all():
                 documents.append(self._convert_sql_row_to_document(row))
@@ -180,24 +217,28 @@ class SQLDocumentStore(BaseDocumentStore):
     def get_all_documents(
         self,
         index: Optional[str] = None,
-        filters: Optional[Dict[str, List[str]]] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
         return_embedding: Optional[bool] = None,
         batch_size: int = 10_000,
-        headers: Optional[Dict[str, str]] = None
+        headers: Optional[Dict[str, str]] = None,
     ) -> List[Document]:
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
-        documents = list(self.get_all_documents_generator(index=index, filters=filters, return_embedding=return_embedding, batch_size=batch_size))
+
+        documents = list(
+            self.get_all_documents_generator(
+                index=index, filters=filters, return_embedding=return_embedding, batch_size=batch_size
+            )
+        )
         return documents
 
     def get_all_documents_generator(
         self,
         index: Optional[str] = None,
-        filters: Optional[Dict[str, List[str]]] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
         return_embedding: Optional[bool] = None,
         batch_size: int = 10_000,
-        headers: Optional[Dict[str, str]] = None
+        headers: Optional[Dict[str, str]] = None,
     ) -> Generator[Document, None, None]:
         """
         Get documents from the document store. Under-the-hood, documents are fetched in batches from the
@@ -213,7 +254,7 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         if return_embedding is True:
             raise Exception("return_embeddings is not supported by SQLDocumentStore.")
         result = self._query(
@@ -222,8 +263,8 @@ class SQLDocumentStore(BaseDocumentStore):
             batch_size=batch_size,
         )
         yield from result
-    
-    def _create_document_field_map(self)->Dict:
+
+    def _create_document_field_map(self) -> Dict:
         """
         There is no field mapping required
         """
@@ -232,10 +273,10 @@ class SQLDocumentStore(BaseDocumentStore):
     def _query(
         self,
         index: Optional[str] = None,
-        filters: Optional[Dict[str, List[str]]] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
         vector_ids: Optional[List[str]] = None,
         only_documents_without_embedding: bool = False,
-        batch_size: int = 10_000
+        batch_size: int = 10_000,
     ):
         """
         :param index: Name of the index to get the documents from. If None, the
@@ -251,20 +292,13 @@ class SQLDocumentStore(BaseDocumentStore):
         # Hence using directly column name improve memory and performance.
         # Refer https://stackoverflow.com/questions/23185319/why-is-loading-sqlalchemy-objects-via-the-orm-5-8x-slower-than-rows-via-a-raw-my
         documents_query = self.session.query(
-            DocumentORM.id,
-            DocumentORM.content,
-            DocumentORM.content_type,
-            DocumentORM.vector_id
+            DocumentORM.id, DocumentORM.content, DocumentORM.content_type, DocumentORM.vector_id
         ).filter_by(index=index)
 
         if filters:
-            for key, values in filters.items():
-                documents_query = documents_query. \
-                    join(MetaDocumentORM, aliased=True). \
-                    filter(
-                        MetaDocumentORM.name == key,
-                        MetaDocumentORM.value.in_(values),
-                    )
+            parsed_filter = LogicalFilterClause.parse(filters)
+            select_ids = parsed_filter.convert_to_sql(MetaDocumentORM)
+            documents_query = documents_query.filter(DocumentORM.id.in_(select_ids))
 
         if only_documents_without_embedding:
             documents_query = documents_query.filter(DocumentORM.vector_id.is_(None))
@@ -277,12 +311,14 @@ class SQLDocumentStore(BaseDocumentStore):
             documents_query = self._windowed_query(documents_query, DocumentORM.id, batch_size)
 
         for i, row in enumerate(documents_query, start=1):
-            documents_map[row.id] = Document.from_dict({
-                "id": row.id,
-                "content": row.content,
-                "content_type": row.content_type,
-                "meta": {} if row.vector_id is None else {"vector_id": row.vector_id}
-            })
+            documents_map[row.id] = Document.from_dict(
+                {
+                    "id": row.id,
+                    "content": row.content,
+                    "content_type": row.content_type,
+                    "meta": {} if row.vector_id is None else {"vector_id": row.vector_id},
+                }
+            )
             if i % batch_size == 0:
                 documents_map = self._get_documents_meta(documents_map)
                 yield from documents_map.values()
@@ -294,9 +330,7 @@ class SQLDocumentStore(BaseDocumentStore):
     def _get_documents_meta(self, documents_map):
         doc_ids = documents_map.keys()
         meta_query = self.session.query(
-         MetaDocumentORM.document_id,
-         MetaDocumentORM.name,
-         MetaDocumentORM.value
+            MetaDocumentORM.document_id, MetaDocumentORM.name, MetaDocumentORM.value
         ).filter(MetaDocumentORM.document_id.in_(doc_ids))
 
         for row in meta_query.all():
@@ -309,7 +343,7 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.label_index
         # TODO: Use batch_size
         label_rows = self.session.query(LabelORM).filter_by(index=index).all()
@@ -317,9 +351,14 @@ class SQLDocumentStore(BaseDocumentStore):
 
         return labels
 
-    def write_documents(self, documents: Union[List[dict], List[Document]], index: Optional[str] = None,
-                        batch_size: int = 10_000, duplicate_documents: Optional[str] = None, 
-                        headers: Optional[Dict[str, str]] = None) -> None:
+    def write_documents(
+        self,
+        documents: Union[List[dict], List[Document]],
+        index: Optional[str] = None,
+        batch_size: int = 10_000,
+        duplicate_documents: Optional[str] = None,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> None:
         """
         Indexes documents for later queries.
 
@@ -342,7 +381,7 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.index
         duplicate_documents = duplicate_documents or self.duplicate_documents
         if len(documents) == 0:
@@ -353,16 +392,22 @@ class SQLDocumentStore(BaseDocumentStore):
         else:
             document_objects = documents
 
-        document_objects = self._handle_duplicate_documents(documents=document_objects,
-                                                            index=index,
-                                                            duplicate_documents=duplicate_documents)
+        document_objects = self._handle_duplicate_documents(
+            documents=document_objects, index=index, duplicate_documents=duplicate_documents
+        )
         for i in range(0, len(document_objects), batch_size):
-            for doc in document_objects[i: i + batch_size]:
+            for doc in document_objects[i : i + batch_size]:
                 meta_fields = doc.meta or {}
                 vector_id = meta_fields.pop("vector_id", None)
                 meta_orms = [MetaDocumentORM(name=key, value=value) for key, value in meta_fields.items()]
-                doc_orm = DocumentORM(id=doc.id, content=doc.to_dict()["content"], content_type=doc.content_type,
-                                      vector_id=vector_id, meta=meta_orms, index=index)
+                doc_orm = DocumentORM(
+                    id=doc.id,
+                    content=doc.to_dict()["content"],
+                    content_type=doc.content_type,
+                    vector_id=vector_id,
+                    meta=meta_orms,
+                    index=index,
+                )
                 if duplicate_documents == "overwrite":
                     # First old meta data cleaning is required
                     self.session.query(MetaDocumentORM).filter_by(document_id=doc.id).delete()
@@ -381,16 +426,18 @@ class SQLDocumentStore(BaseDocumentStore):
         """Write annotation labels into document store."""
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         labels = [Label.from_dict(l) if isinstance(l, dict) else l for l in labels]
         index = index or self.label_index
 
         duplicate_ids: list = [label.id for label in self._get_duplicate_labels(labels, index=index)]
         if len(duplicate_ids) > 0:
-            logger.warning(f"Duplicate Label IDs: Inserting a Label whose id already exists in this document store."
-                           f" This will overwrite the old Label. Please make sure Label.id is a unique identifier of"
-                           f" the answer annotation and not the question."
-                           f"   Problematic ids: {','.join(duplicate_ids)}")
+            logger.warning(
+                f"Duplicate Label IDs: Inserting a Label whose id already exists in this document store."
+                f" This will overwrite the old Label. Please make sure Label.id is a unique identifier of"
+                f" the answer annotation and not the question."
+                f"   Problematic ids: {','.join(duplicate_ids)}"
+            )
         # TODO: Use batch_size
 
         for label in labels:
@@ -420,7 +467,7 @@ class SQLDocumentStore(BaseDocumentStore):
             else:
                 self.session.add(label_orm)
 
-            #TODO: investigate why test_multilabel() failed when not committing within the loop
+            # TODO: investigate why test_multilabel() failed when not committing within the loop
             # Seems that in some cases only the last label get than "committed"
             self.session.commit()
 
@@ -434,15 +481,15 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         index = index or self.index
         for chunk_map in self.chunked_dict(vector_id_map, size=batch_size):
-            self.session.query(DocumentORM).filter(
-                DocumentORM.id.in_(chunk_map),
-                DocumentORM.index == index
-            ).update({
-                DocumentORM.vector_id: case(
-                    chunk_map,
-                    value=DocumentORM.id,
-                )
-            }, synchronize_session=False)
+            self.session.query(DocumentORM).filter(DocumentORM.id.in_(chunk_map), DocumentORM.index == index).update(
+                {
+                    DocumentORM.vector_id: case(
+                        chunk_map,
+                        value=DocumentORM.id,
+                    )
+                },
+                synchronize_session=False,
+            )
             try:
                 self.session.commit()
             except Exception as ex:
@@ -465,31 +512,35 @@ class SQLDocumentStore(BaseDocumentStore):
         if not index:
             index = self.index
         self.session.query(MetaDocumentORM).filter_by(document_id=id, document_index=index).delete()
-        meta_orms = [MetaDocumentORM(name=key, value=value, document_id=id, document_index=index) for key, value in meta.items()]
+        meta_orms = [
+            MetaDocumentORM(name=key, value=value, document_id=id, document_index=index) for key, value in meta.items()
+        ]
         for m in meta_orms:
             self.session.add(m)
         self.session.commit()
 
-    def get_document_count(self, filters: Optional[Dict[str, List[str]]] = None, index: Optional[str] = None, 
-            only_documents_without_embedding: bool = False, 
-            headers: Optional[Dict[str, str]] = None) -> int:
+    def get_document_count(
+        self,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
+        index: Optional[str] = None,
+        only_documents_without_embedding: bool = False,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> int:
         """
         Return the number of documents in the document store.
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.index
         query = self.session.query(DocumentORM).filter_by(index=index)
 
         if filters:
             for key, values in filters.items():
-                query = query. \
-                    join(MetaDocumentORM, aliased=True). \
-                    filter(
-                        MetaDocumentORM.name == key,
-                        MetaDocumentORM.value.in_(values),
-                    )
+                query = query.join(MetaDocumentORM, aliased=True).filter(
+                    MetaDocumentORM.name == key,
+                    MetaDocumentORM.value.in_(values),
+                )
 
         if only_documents_without_embedding:
             query = query.filter(DocumentORM.vector_id.is_(None))
@@ -503,7 +554,7 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.label_index
         return self.session.query(LabelORM).filter_by(index=index).count()
 
@@ -512,7 +563,7 @@ class SQLDocumentStore(BaseDocumentStore):
             "id": row.id,
             "content": row.content,
             "content_type": row.content_type,
-            "meta": {meta.name: meta.value for meta in row.meta}
+            "meta": {meta.name: meta.value for meta in row.meta},
         }
         document = Document.from_dict(doc_dict)
 
@@ -525,7 +576,7 @@ class SQLDocumentStore(BaseDocumentStore):
 
         label = Label(
             query=row.query,
-            answer=Answer.from_json(row.answer), #type: ignore
+            answer=Answer.from_json(row.answer),  # type: ignore
             document=Document.from_json(row.document),
             is_correct_answer=row.is_correct_answer,
             is_correct_document=row.is_correct_document,
@@ -535,23 +586,32 @@ class SQLDocumentStore(BaseDocumentStore):
             pipeline_id=row.pipeline_id,
             created_at=row.created_at,
             updated_at=row.updated_at,
-            meta=row.meta
+            meta=row.meta,
         )
         return label
-        
-    def query_by_embedding(self,
-                           query_emb: np.ndarray,
-                           filters: Optional[dict] = None,
-                           top_k: int = 10,
-                           index: Optional[str] = None,
-                           return_embedding: Optional[bool] = None,
-                           headers: Optional[Dict[str, str]] = None) -> List[Document]:
 
-        raise NotImplementedError("SQLDocumentStore is currently not supporting embedding queries. "
-                                  "Change the query type (e.g. by choosing a different retriever) "
-                                  "or change the DocumentStore (e.g. to ElasticsearchDocumentStore)")
+    def query_by_embedding(
+        self,
+        query_emb: np.ndarray,
+        filters: Optional[dict] = None,
+        top_k: int = 10,
+        index: Optional[str] = None,
+        return_embedding: Optional[bool] = None,
+        headers: Optional[Dict[str, str]] = None,
+    ) -> List[Document]:
 
-    def delete_all_documents(self, index: Optional[str] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
+        raise NotImplementedError(
+            "SQLDocumentStore is currently not supporting embedding queries. "
+            "Change the query type (e.g. by choosing a different retriever) "
+            "or change the DocumentStore (e.g. to ElasticsearchDocumentStore)"
+        )
+
+    def delete_all_documents(
+        self,
+        index: Optional[str] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
+        headers: Optional[Dict[str, str]] = None,
+    ):
         """
         Delete documents in an index. All documents are deleted if no filters are passed.
 
@@ -561,16 +621,22 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         logger.warning(
-                """DEPRECATION WARNINGS: 
+            """DEPRECATION WARNINGS: 
                 1. delete_all_documents() method is deprecated, please use delete_documents method
                 For more details, please refer to the issue: https://github.com/deepset-ai/haystack/issues/1045
                 """
         )
         self.delete_documents(index, None, filters)
 
-    def delete_documents(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
+    def delete_documents(
+        self,
+        index: Optional[str] = None,
+        ids: Optional[List[str]] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
+        headers: Optional[Dict[str, str]] = None,
+    ):
         """
         Delete documents in an index. All documents are deleted if no filters are passed.
 
@@ -588,24 +654,29 @@ class SQLDocumentStore(BaseDocumentStore):
         if not filters and not ids:
             self.session.query(DocumentORM).filter_by(index=index).delete(synchronize_session=False)
         else:
-            document_ids_to_delete = self.session.query(DocumentORM.id).filter(DocumentORM.index==index)
+            document_ids_to_delete = self.session.query(DocumentORM.id).filter(DocumentORM.index == index)
             if filters:
                 for key, values in filters.items():
-                    document_ids_to_delete = document_ids_to_delete. \
-                        join(MetaDocumentORM, aliased=True). \
-                        filter(
-                            MetaDocumentORM.name == key,
-                            MetaDocumentORM.value.in_(values),
-                        )
+                    document_ids_to_delete = document_ids_to_delete.join(MetaDocumentORM, aliased=True).filter(
+                        MetaDocumentORM.name == key,
+                        MetaDocumentORM.value.in_(values),
+                    )
             if ids:
                 document_ids_to_delete = document_ids_to_delete.filter(DocumentORM.id.in_(ids))
 
             self.session.query(DocumentORM).filter(DocumentORM.id.in_(document_ids_to_delete)).delete(
-                    synchronize_session=False)
+                synchronize_session=False
+            )
 
         self.session.commit()
 
-    def delete_labels(self, index: Optional[str] = None, ids: Optional[List[str]] = None, filters: Optional[Dict[str, List[str]]] = None, headers: Optional[Dict[str, str]] = None):
+    def delete_labels(
+        self,
+        index: Optional[str] = None,
+        ids: Optional[List[str]] = None,
+        filters: Optional[Dict[str, Any]] = None,  # TODO: Adapt type once we allow extended filters in SQLDocStore
+        headers: Optional[Dict[str, str]] = None,
+    ):
         """
         Delete labels from the document store. All labels are deleted if no filters are passed.
 
@@ -618,7 +689,7 @@ class SQLDocumentStore(BaseDocumentStore):
         """
         if headers:
             raise NotImplementedError("SQLDocumentStore does not support headers.")
-        
+
         index = index or self.label_index
         if not filters and not ids:
             self.session.query(LabelORM).filter_by(index=index).delete(synchronize_session=False)
@@ -627,15 +698,12 @@ class SQLDocumentStore(BaseDocumentStore):
             if filters:
                 for key, values in filters.items():
                     label_attribute = getattr(LabelORM, key)
-                    label_ids_to_delete = label_ids_to_delete.filter(
-                        label_attribute.in_(values)
-                    )
+                    label_ids_to_delete = label_ids_to_delete.filter(label_attribute.in_(values))
 
             if ids:
                 label_ids_to_delete = label_ids_to_delete.filter(LabelORM.id.in_(ids))
 
-            self.session.query(LabelORM).filter(LabelORM.id.in_(label_ids_to_delete)).delete(
-                synchronize_session=False)
+            self.session.query(LabelORM).filter(LabelORM.id.in_(label_ids_to_delete)).delete(synchronize_session=False)
 
         self.session.commit()
 
@@ -666,20 +734,11 @@ class SQLDocumentStore(BaseDocumentStore):
 
         def int_for_range(start_id, end_id):
             if end_id:
-                return and_(
-                    column >= start_id,
-                    column < end_id
-                )
+                return and_(column >= start_id, column < end_id)
             else:
                 return column >= start_id
 
-        q = session.query(
-            column,
-            func.row_number(). \
-                over(order_by=column). \
-                label('rownum')
-        ). \
-            from_self(column)
+        q = session.query(column, func.row_number().over(order_by=column).label("rownum")).from_self(column)
         if windowsize > 1:
             q = q.filter(text("rownum %% %d=1" % windowsize))
 
@@ -694,10 +753,8 @@ class SQLDocumentStore(BaseDocumentStore):
             yield int_for_range(start, end)
 
     def _windowed_query(self, q, column, windowsize):
-        """"Break a Query into windows on a given column."""
+        """ "Break a Query into windows on a given column."""
 
-        for whereclause in self._column_windows(
-                q.session,
-                column, windowsize):
+        for whereclause in self._column_windows(q.session, column, windowsize):
             for row in q.filter(whereclause).order_by(column):
                 yield row
