@@ -26,11 +26,12 @@ class BaseAdaptiveModel:
     """
     Base Class for implementing AdaptiveModel with frameworks like PyTorch and ONNX.
     """
+
     language_model: LanguageModel
     subclasses = {}  # type: Dict
 
     def __init_subclass__(cls, **kwargs):
-        """ 
+        """
         This automatically keeps track of all available subclasses.
         Enables generic load() for all specific AdaptiveModel implementation.
         """
@@ -120,7 +121,7 @@ class BaseAdaptiveModel:
             head.label_tensor_name = tasks[head.task_name]["label_tensor_name"]
             label_list = tasks[head.task_name]["label_list"]
             if not label_list and require_labels:
-                raise Exception(f"The task \'{head.task_name}\' is missing a valid set of labels")
+                raise Exception(f"The task '{head.task_name}' is missing a valid set of labels")
             label_list = tasks[head.task_name]["label_list"]
             head.label_list = label_list
             head.metric = tasks[head.task_name]["metric"]
@@ -129,16 +130,8 @@ class BaseAdaptiveModel:
     def _get_prediction_head_files(cls, load_dir: Union[str, Path], strict: bool = True):
         load_dir = Path(load_dir)
         files = os.listdir(load_dir)
-        model_files = [
-            load_dir / f
-            for f in files
-            if ".bin" in f and "prediction_head" in f
-        ]
-        config_files = [
-            load_dir / f
-            for f in files
-            if "config.json" in f and "prediction_head" in f
-        ]
+        model_files = [load_dir / f for f in files if ".bin" in f and "prediction_head" in f]
+        config_files = [load_dir / f for f in files if "config.json" in f and "prediction_head" in f]
         # sort them to get correct order in case of multiple prediction heads
         model_files.sort()
         config_files.sort()
@@ -154,6 +147,7 @@ class BaseAdaptiveModel:
 
         return model_files, config_files
 
+
 def loss_per_head_sum(loss_per_head: Iterable, global_step: Optional[int] = None, batch: Optional[Dict] = None):
     """
     Sums up the loss of each prediction head.
@@ -164,10 +158,11 @@ def loss_per_head_sum(loss_per_head: Iterable, global_step: Optional[int] = None
 
 
 class AdaptiveModel(nn.Module, BaseAdaptiveModel):
-    """ 
+    """
     PyTorch implementation containing all the modelling needed for your NLP task. Combines a language
     model and a prediction head. Allows for gradient flow back to the language model component.
     """
+
     def __init__(
         self,
         language_model: LanguageModel,
@@ -206,9 +201,7 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         self.prediction_heads = nn.ModuleList([ph.to(device) for ph in prediction_heads])
         self.fit_heads_to_lm()
         self.dropout = nn.Dropout(embeds_dropout_prob)
-        self.lm_output_types = (
-            [lm_output_types] if isinstance(lm_output_types, str) else lm_output_types
-        )
+        self.lm_output_types = [lm_output_types] if isinstance(lm_output_types, str) else lm_output_types
         self.log_params()
         # default loss aggregation function is a simple sum (without using any of the optional params)
         if not loss_aggregation_fn:
@@ -265,8 +258,14 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
             # Need to save config and pipeline
 
     @classmethod
-    def load(cls, load_dir: Union[str, Path], device: str, strict: bool = True, lm_name: Optional[str] = None,  #  type: ignore
-             processor: Optional[Processor] = None):
+    def load(  #  type: ignore
+        cls,
+        load_dir: Union[str, Path],  #  type: ignore
+        device: str,  #  type: ignore
+        strict: bool = True,  #  type: ignore
+        lm_name: Optional[str] = None,  #  type: ignore
+        processor: Optional[Processor] = None,  #  type: ignore
+    ):
         """
         Loads an AdaptiveModel from a directory. The directory must contain:
 
@@ -316,10 +315,11 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         all_losses = []
         for head, logits_for_one_head in zip(self.prediction_heads, logits):
             # check if PredictionHead connected to Processor
-            assert hasattr(head, "label_tensor_name"), \
-                (f"Label_tensor_names are missing inside the {head.task_name} Prediction Head. Did you connect the model"
+            assert hasattr(head, "label_tensor_name"), (
+                f"Label_tensor_names are missing inside the {head.task_name} Prediction Head. Did you connect the model"
                 " with the processor through either 'model.connect_heads_with_processor(processor.tasks)'"
-                " or by passing the processor to the Adaptive Model?")
+                " or by passing the processor to the Adaptive Model?"
+            )
             all_losses.append(head.logits_to_loss(logits=logits_for_one_head, **kwargs))
         return all_losses
 
@@ -368,7 +368,9 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         :return: All logits as torch.tensor or multiple tensors.
         """
         # Run forward pass of language model
-        output_tuple = self.language_model.forward(**kwargs, output_hidden_states=output_hidden_states, output_attentions=output_attentions)
+        output_tuple = self.language_model.forward(
+            **kwargs, output_hidden_states=output_hidden_states, output_attentions=output_attentions
+        )
         if output_hidden_states:
             if output_attentions:
                 sequence_output, pooled_output, hidden_states, attentions = output_tuple
@@ -393,9 +395,7 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
                 ):  # we need a per_token_squad because of variable metric computation later on...
                     output = self.dropout(sequence_output)
                 else:
-                    raise ValueError(
-                        "Unknown extraction strategy from language model: {}".format(lm_out)
-                    )
+                    raise ValueError("Unknown extraction strategy from language model: {}".format(lm_out))
 
                 # Do the actual forward pass of a single head
                 all_logits.append(head(output))
@@ -427,13 +427,15 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
 
         # Run forward pass of language model
         if extraction_layer == -1:
-            sequence_output, pooled_output = self.language_model(**kwargs, return_dict=False, output_all_encoded_layers=False)
+            sequence_output, pooled_output = self.language_model(
+                **kwargs, return_dict=False, output_all_encoded_layers=False
+            )
         else:
             # get output from an earlier layer
             self.language_model.enable_hidden_states_output()
             sequence_output, pooled_output, all_hidden_states = self.language_model(**kwargs, return_dict=False)
             sequence_output = all_hidden_states[extraction_layer]
-            pooled_output = None #not available in earlier layers
+            pooled_output = None  # not available in earlier layers
             self.language_model.disable_hidden_states_output()
         return sequence_output, pooled_output
 
@@ -444,9 +446,7 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         params = {
             "lm_type": self.language_model.__class__.__name__,
             "lm_name": self.language_model.name,
-            "prediction_heads": ",".join(
-                [head.__class__.__name__ for head in self.prediction_heads]
-            ),
+            "prediction_heads": ",".join([head.__class__.__name__ for head in self.prediction_heads]),
             "lm_output_types": ",".join(self.lm_output_types),
         }
         try:
@@ -461,9 +461,11 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         """
         model_vocab_len = self.language_model.model.resize_token_embeddings(new_num_tokens=None).num_embeddings
 
-        msg = f"Vocab size of tokenizer {vocab_size} doesn't match with model {model_vocab_len}. " \
-              "If you added a custom vocabulary to the tokenizer, " \
-              "make sure to supply 'n_added_tokens' to LanguageModel.load() and BertStyleLM.load()"
+        msg = (
+            f"Vocab size of tokenizer {vocab_size} doesn't match with model {model_vocab_len}. "
+            "If you added a custom vocabulary to the tokenizer, "
+            "make sure to supply 'n_added_tokens' to LanguageModel.load() and BertStyleLM.load()"
+        )
         assert vocab_size == model_vocab_len, msg
 
         for head in self.prediction_heads:
@@ -484,8 +486,16 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         return conv.Converter.convert_to_transformers(self)
 
     @classmethod
-    def convert_from_transformers(cls, model_name_or_path: Union[str, Path], device: str, revision: Optional[str] = None,
-                                  task_type: Optional[str] = None, processor: Optional[Processor] = None,  use_auth_token: Optional[Union[bool, str]] = None, **kwargs):
+    def convert_from_transformers(
+        cls,
+        model_name_or_path: Union[str, Path],
+        device: str,
+        revision: Optional[str] = None,
+        task_type: Optional[str] = None,
+        processor: Optional[Processor] = None,
+        use_auth_token: Optional[Union[bool, str]] = None,
+        **kwargs,
+    ):
         """
         Load a (downstream) model from huggingface's transformers format. Use cases:
          - continue training in Haystack (e.g. take a squad QA model and fine-tune on your own data)
@@ -507,18 +517,26 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
         :type processor: Processor
         :return: AdaptiveModel
         """
-        return conv.Converter.convert_from_transformers(model_name_or_path,
-                                                        revision=revision,
-                                                        device=device,
-                                                        task_type=task_type,
-                                                        processor=processor,
-                                                        use_auth_token=use_auth_token,
-                                                        **kwargs)
-
+        return conv.Converter.convert_from_transformers(
+            model_name_or_path,
+            revision=revision,
+            device=device,
+            task_type=task_type,
+            processor=processor,
+            use_auth_token=use_auth_token,
+            **kwargs,
+        )
 
     @classmethod
-    def convert_to_onnx(cls, model_name: str, output_path: Path, task_type: str, convert_to_float16: bool = False,
-                        quantize: bool = False, opset_version: int = 11):
+    def convert_to_onnx(
+        cls,
+        model_name: str,
+        output_path: Path,
+        task_type: str,
+        convert_to_float16: bool = False,
+        quantize: bool = False,
+        opset_version: int = 11,
+    ):
         """
         Convert a PyTorch model from transformers hub to an ONNX Model.
 
@@ -544,18 +562,14 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
             pipeline_name=task_type_to_pipeline_map[task_type],
             framework="pt",
             model=model_name,
-            output=output_path/"model.onnx",
+            output=output_path / "model.onnx",
             opset=opset_version,
-            use_external_format=True if language_model_class=="XLMRoberta" else False
+            use_external_format=True if language_model_class == "XLMRoberta" else False,
         )
 
         # save processor & model config files that are needed when loading the model with the Haystack.basics Inferencer
         processor = Processor.convert_from_transformers(
-            tokenizer_name_or_path=model_name,
-            task_type=task_type,
-            max_seq_len=256,
-            doc_stride=128,
-            use_fast=True
+            tokenizer_name_or_path=model_name, task_type=task_type, max_seq_len=256, doc_stride=128, use_fast=True
         )
         processor.save(output_path)
         model = AdaptiveModel.convert_from_transformers(model_name, device="cpu", task_type=task_type)
@@ -566,25 +580,26 @@ class AdaptiveModel(nn.Module, BaseAdaptiveModel):
             "task_type": task_type,
             "onnx_opset_version": opset_version,
             "language_model_class": language_model_class,
-            "language": model.language_model.language
+            "language": model.language_model.language,
         }
         with open(output_path / "onnx_model_config.json", "w") as f:
             json.dump(onnx_model_config, f)
 
         if convert_to_float16:
             from onnxruntime_tools import optimizer
+
             config = AutoConfig.from_pretrained(model_name)
             optimized_model = optimizer.optimize_model(
-                input=str(output_path/"model.onnx"),
-                model_type='bert',
+                input=str(output_path / "model.onnx"),
+                model_type="bert",
                 num_heads=config.num_hidden_layers,
-                hidden_size=config.hidden_size
+                hidden_size=config.hidden_size,
             )
             optimized_model.convert_model_float32_to_float16()
             optimized_model.save_model_to_file("model.onnx")
 
         if quantize:
-            quantize_model(output_path/"model.onnx")
+            quantize_model(output_path / "model.onnx")
 
 
 class ONNXAdaptiveModel(BaseAdaptiveModel):
@@ -596,14 +611,15 @@ class ONNXAdaptiveModel(BaseAdaptiveModel):
 
     For inference, this class is compatible with the Haystack.basics Inferencer.
     """
+
     # TODO validate usefulness
     def __init__(
         self,
-        onnx_session, # TODO
+        onnx_session,  # TODO
         language_model_class: str,
         language: str,
         prediction_heads: List[PredictionHead],
-        device: str
+        device: str,
     ):
         """
         :param onnx_session: ? # TODO
@@ -613,9 +629,14 @@ class ONNXAdaptiveModel(BaseAdaptiveModel):
         :param device: The device on which this model will operate. Either "cpu" or "cuda".
         """
         import onnxruntime
+
+        super().__init__(prediction_heads)
+
         if str(device) == "cuda" and onnxruntime.get_device() != "GPU":
-            raise Exception(f"Device {device} not available for Inference. For CPU, run pip install onnxruntime and"
-                            f"for GPU run pip install onnxruntime-gpu")
+            raise Exception(
+                f"Device {device} not available for Inference. For CPU, run pip install onnxruntime and"
+                f"for GPU run pip install onnxruntime-gpu"
+            )
         self.onnx_session = onnx_session
         self.language_model_class = language_model_class
         self.language = language
@@ -632,6 +653,7 @@ class ONNXAdaptiveModel(BaseAdaptiveModel):
         """
         load_dir = Path(load_dir)
         import onnxruntime
+
         sess_options = onnxruntime.SessionOptions()
         # Set graph optimization level to ORT_ENABLE_EXTENDED to enable bert optimization.
         sess_options.graph_optimization_level = onnxruntime.GraphOptimizationLevel.ORT_ENABLE_EXTENDED
@@ -650,7 +672,7 @@ class ONNXAdaptiveModel(BaseAdaptiveModel):
             prediction_heads.append(head)
             ph_output_type.append(head.ph_output_type)
 
-        with open(load_dir/"onnx_model_config.json") as f:
+        with open(load_dir / "onnx_model_config.json") as f:
             model_config = json.load(f)
             language_model_class = model_config["language_model_class"]
             language = model_config["language"]
@@ -667,14 +689,14 @@ class ONNXAdaptiveModel(BaseAdaptiveModel):
         with torch.no_grad():
             if self.language_model_class == "Bert":
                 input_to_onnx = {
-                    'input_ids': numpy.ascontiguousarray(kwargs['input_ids'].cpu().numpy()),
-                    'attention_mask': numpy.ascontiguousarray(kwargs['padding_mask'].cpu().numpy()),
-                    'token_type_ids': numpy.ascontiguousarray(kwargs['segment_ids'].cpu().numpy()),
+                    "input_ids": numpy.ascontiguousarray(kwargs["input_ids"].cpu().numpy()),
+                    "attention_mask": numpy.ascontiguousarray(kwargs["padding_mask"].cpu().numpy()),
+                    "token_type_ids": numpy.ascontiguousarray(kwargs["segment_ids"].cpu().numpy()),
                 }
             elif self.language_model_class in ["Roberta", "XLMRoberta"]:
                 input_to_onnx = {
-                    'input_ids': numpy.ascontiguousarray(kwargs['input_ids'].cpu().numpy()),
-                    'attention_mask': numpy.ascontiguousarray(kwargs['padding_mask'].cpu().numpy())
+                    "input_ids": numpy.ascontiguousarray(kwargs["input_ids"].cpu().numpy()),
+                    "attention_mask": numpy.ascontiguousarray(kwargs["padding_mask"].cpu().numpy()),
                 }
             res = self.onnx_session.run(None, input_to_onnx)
             res = numpy.stack(res).transpose(1, 2, 0)
@@ -704,6 +726,7 @@ class ONNXWrapper(AdaptiveModel):
     to the forward pass of the model. However, the AdaptiveModel's forward takes keyword arguments.
     This class circumvents the issue by converting positional arguments to keyword arguments.
     """
+
     @classmethod
     def load_from_adaptive_model(cls, adaptive_model: AdaptiveModel):
         model = copy.deepcopy(adaptive_model)
