@@ -1,4 +1,4 @@
-#TODO analyse if this optimization is needed or whether we can use HF transformers code
+# TODO analyse if this optimization is needed or whether we can use HF transformers code
 from typing import Dict, Any
 
 import inspect
@@ -9,33 +9,45 @@ import torch
 from torch.nn import DataParallel
 from torch.nn.parallel import DistributedDataParallel
 
+logger = logging.getLogger(__name__)
+
 try:
-    from apex import amp
+    from apex import amp  # pylint: disable=import-error
+
+    logger.info("apex is available.")
+
     try:
-        from apex.parallel import convert_syncbn_model
+        from apex.parallel import convert_syncbn_model  # pylint: disable=import-error
+
         APEX_PARALLEL_AVAILABLE = True
+
+        logger.info("apex.parallel is available.")
+
     except AttributeError:
         APEX_PARALLEL_AVAILABLE = False
+        logger.info("apex.parallel not found, won't use it." "See https://nvidia.github.io/apex/parallel.html")
+
     AMP_AVAILABLE = True
+
 except ImportError:
     AMP_AVAILABLE = False
     APEX_PARALLEL_AVAILABLE = False
+    logger.info("apex not found, won't use it. " "See https://nvidia.github.io/apex/")
+
 
 from haystack.modeling.model.adaptive_model import AdaptiveModel
 from haystack.modeling.logger import MLFlowLogger as MlLogger
 
 
-logger = logging.getLogger(__name__)
-
-
 class WrappedDataParallel(DataParallel):
     """
-    A way of adapting attributes of underlying class to parallel mode. See: 
+    A way of adapting attributes of underlying class to parallel mode. See:
     https://pytorch.org/tutorials/beginner/former_torchies/parallelism_tutorial.html#dataparallel
 
-    Gets into recursion errors. Workaround see: 
+    Gets into recursion errors. Workaround see:
     https://discuss.pytorch.org/t/access-att-of-model-wrapped-within-torch-nn-dataparallel-maximum-recursion-depth-exceeded/46975
     """
+
     def __getattr__(self, name):
         try:
             return super().__getattr__(name)
@@ -49,6 +61,7 @@ class WrappedDDP(DistributedDataParallel):
     Even when using distributed on a single machine with multiple GPUs, apex can speed up training significantly.
     Distributed code must be launched with "python -m torch.distributed.launch --nproc_per_node=1 run_script.py"
     """
+
     def __getattr__(self, name):
         try:
             return super().__getattr__(name)
@@ -56,17 +69,19 @@ class WrappedDDP(DistributedDataParallel):
             return getattr(self.module, name)
 
 
-def initialize_optimizer(model: AdaptiveModel,
-                         n_batches: int,
-                         n_epochs: int,
-                         device,
-                         learning_rate: float,
-                         optimizer_opts: Dict[Any, Any] = None,
-                         schedule_opts: Dict[Any, Any] = None,
-                         distributed: bool = False,
-                         grad_acc_steps: int = 1,
-                         local_rank: int = -1,
-                         use_amp: str = None):
+def initialize_optimizer(
+    model: AdaptiveModel,
+    n_batches: int,
+    n_epochs: int,
+    device,
+    learning_rate: float,
+    optimizer_opts: Dict[Any, Any] = None,
+    schedule_opts: Dict[Any, Any] = None,
+    distributed: bool = False,
+    grad_acc_steps: int = 1,
+    local_rank: int = -1,
+    use_amp: str = None,
+):
     """
     Initializes an optimizer, a learning rate scheduler and converts the model if needed (e.g for mixed precision).
     Per default, we use transformers' AdamW and a linear warmup schedule with warmup ratio 0.1.
@@ -115,13 +130,16 @@ def initialize_optimizer(model: AdaptiveModel,
     :return: model, optimizer, scheduler
     """
     if use_amp and not AMP_AVAILABLE:
-        raise ImportError(f'Got use_amp = {use_amp}, but cannot find apex. '
-                          'Please install Apex if you want to make use of automatic mixed precision. '
-                          'https://github.com/NVIDIA/apex')
+        raise ImportError(
+            f"Got use_amp = {use_amp}, but cannot find apex. "
+            "Please install Apex if you want to make use of automatic mixed precision. "
+            "https://github.com/NVIDIA/apex"
+        )
 
     if (schedule_opts is not None) and (not isinstance(schedule_opts, dict)):
-        raise TypeError('Parameter schedule_opts must be None or '
-                        'an instance of dict but was {}!'.format(type(schedule_opts)))
+        raise TypeError(
+            "Parameter schedule_opts must be None or " "an instance of dict but was {}!".format(type(schedule_opts))
+        )
 
     num_train_optimization_steps = int(n_batches / grad_acc_steps) * n_epochs
 
@@ -132,9 +150,11 @@ def initialize_optimizer(model: AdaptiveModel,
 
     if schedule_opts is None:
         # Default schedule: Linear Warmup with 10% warmup
-        schedule_opts = {"name": "LinearWarmup",
-                         "num_warmup_steps": 0.1 * num_train_optimization_steps,
-                         "num_training_steps": num_train_optimization_steps}
+        schedule_opts = {
+            "name": "LinearWarmup",
+            "num_warmup_steps": 0.1 * num_train_optimization_steps,
+            "num_training_steps": num_train_optimization_steps,
+        }
 
         # schedule_opts = {"name": "OneCycleLR", "max_lr":learning_rate, "pct_start": 0.1,
         #                  "total_steps": num_train_optimization_steps }
@@ -142,10 +162,12 @@ def initialize_optimizer(model: AdaptiveModel,
         schedule_opts["num_training_steps"] = num_train_optimization_steps
 
     # Log params
-    MlLogger.log_params({
-         "use_amp": use_amp,
-         "num_train_optimization_steps": schedule_opts["num_training_steps"],
-        })
+    MlLogger.log_params(
+        {
+            "use_amp": use_amp,
+            "num_train_optimization_steps": schedule_opts["num_training_steps"],
+        }
+    )
 
     # Get optimizer from pytorch, transformers or apex
     optimizer = _get_optim(model, optimizer_opts)
@@ -169,41 +191,49 @@ def _get_optim(model, opts: Dict):
     will have weight_decay set to 0.
     :return: created optimizer
     """
-    optimizer_name = opts.pop('name', None)
+    optimizer_name = opts.pop("name", None)
 
     # Logging
     logger.info(f"Loading optimizer `{optimizer_name}`: '{opts}'")
     MlLogger.log_params(opts)
     MlLogger.log_params({"optimizer_name": optimizer_name})
 
-    weight_decay = opts.pop('weight_decay', None)
-    no_decay = opts.pop('no_decay', None)
+    weight_decay = opts.pop("weight_decay", None)
+    no_decay = opts.pop("no_decay", None)
 
     if no_decay:
         optimizable_parameters = [
-            {'params': [p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay) and p.requires_grad],
-             **opts},
-            {'params': [p for n, p in model.named_parameters() if any(nd in n for nd in no_decay) and p.requires_grad],
-             'weight_decay': 0.0,
-             **opts}
+            {
+                "params": [
+                    p for n, p in model.named_parameters() if not any(nd in n for nd in no_decay) and p.requires_grad
+                ],
+                **opts,
+            },
+            {
+                "params": [
+                    p for n, p in model.named_parameters() if any(nd in n for nd in no_decay) and p.requires_grad
+                ],
+                "weight_decay": 0.0,
+                **opts,
+            },
         ]
     else:
-        optimizable_parameters = [{'params': [p for p in model.parameters() if p.requires_grad], **opts}]
+        optimizable_parameters = [{"params": [p for p in model.parameters() if p.requires_grad], **opts}]
 
     # default weight decay is not the same for all optimizers, so we can't use default value
     # only explicitly add weight decay if it's given
     if weight_decay is not None:
-        optimizable_parameters[0]['weight_decay'] = weight_decay  # type: ignore
+        optimizable_parameters[0]["weight_decay"] = weight_decay  # type: ignore
 
     # Import optimizer by checking in order: torch, transformers, apex and local imports
     try:
-        optim_constructor = getattr(import_module('torch.optim'), optimizer_name)
+        optim_constructor = getattr(import_module("torch.optim"), optimizer_name)
     except AttributeError:
         try:
-            optim_constructor = getattr(import_module('transformers.optimization'), optimizer_name)
+            optim_constructor = getattr(import_module("transformers.optimization"), optimizer_name)
         except AttributeError:
             try:
-                optim_constructor = getattr(import_module('apex.optimizers'), optimizer_name)
+                optim_constructor = getattr(import_module("apex.optimizers"), optimizer_name)
             except (AttributeError, ImportError):
                 try:
                     # Workaround to allow loading AdamW from transformers
@@ -211,36 +241,39 @@ def _get_optim(model, opts: Dict):
                     # which is done in the original BERT implementation)
                     optim_constructor = getattr(sys.modules[__name__], optimizer_name)
                 except (AttributeError, ImportError):
-                    raise AttributeError(f"Optimizer '{optimizer_name}' not found in 'torch', 'transformers', 'apex' or 'local imports")
+                    raise AttributeError(
+                        f"Optimizer '{optimizer_name}' not found in 'torch', 'transformers', 'apex' or 'local imports"
+                    )
 
     return optim_constructor(optimizable_parameters)
 
 
 def get_scheduler(optimizer, opts):
-    """ 
+    """
     Get the scheduler based on dictionary with options. Options are passed to the scheduler constructor.
 
     :param optimizer: optimizer whose learning rate to control
     :param opts: dictionary of args to be passed to constructor of schedule
     :return: created scheduler
     """
-    schedule_name = opts.get('name')
+    schedule_name = opts.get("name")
     try:
-        sched_constructor = getattr(import_module('torch.optim.lr_scheduler'), schedule_name)
+        sched_constructor = getattr(import_module("torch.optim.lr_scheduler"), schedule_name)
     except AttributeError:
         try:
             # The method names in transformers became quite long and unhandy.
             # for convenience we offer usage of shorter alias (e.g. "LinearWarmup")
-            scheduler_translations = {"LinearWarmup": "get_linear_schedule_with_warmup",
-                                      "ConstantWarmup": "get_constant_schedule_with_warmup",
-                                      "Constant": "get_constant_schedule",
-                                      "CosineWarmup": "get_cosine_schedule_with_warmup",
-                                     "CosineWarmupWithRestarts": "get_cosine_with_hard_restarts_schedule_with_warmup"
+            scheduler_translations = {
+                "LinearWarmup": "get_linear_schedule_with_warmup",
+                "ConstantWarmup": "get_constant_schedule_with_warmup",
+                "Constant": "get_constant_schedule",
+                "CosineWarmup": "get_cosine_schedule_with_warmup",
+                "CosineWarmupWithRestarts": "get_cosine_with_hard_restarts_schedule_with_warmup",
             }
             if schedule_name in scheduler_translations.keys():
                 schedule_name = scheduler_translations[schedule_name]
             # in contrast to torch, we actually get here a method and not a class
-            sched_constructor = getattr(import_module('transformers.optimization'), schedule_name)
+            sched_constructor = getattr(import_module("transformers.optimization"), schedule_name)
         except AttributeError:
             raise AttributeError(f"Scheduler '{schedule_name}' not found in 'torch' or 'transformers'")
 
@@ -250,8 +283,8 @@ def get_scheduler(optimizer, opts):
     allowed_args = inspect.signature(sched_constructor).parameters.keys()
 
     # convert from warmup proportion to steps if required
-    if 'num_warmup_steps' in allowed_args and 'num_warmup_steps' not in opts and 'warmup_proportion' in opts:
-        opts['num_warmup_steps'] = int(opts["warmup_proportion"] * opts["num_training_steps"])
+    if "num_warmup_steps" in allowed_args and "num_warmup_steps" not in opts and "warmup_proportion" in opts:
+        opts["num_warmup_steps"] = int(opts["warmup_proportion"] * opts["num_training_steps"])
         MlLogger.log_params({"warmup_proportion": opts["warmup_proportion"]})
 
     # only pass args that are supported by the constructor
@@ -297,10 +330,7 @@ def optimize_model(model, device, local_rank, optimizer=None, distributed=False,
 
         # for some models DistributedDataParallel might complain about parameters
         # not contributing to loss. find_used_parameters remedies that.
-        model = WrappedDDP(model,
-                           device_ids=[local_rank],
-                           output_device=local_rank,
-                           find_unused_parameters=True)
+        model = WrappedDDP(model, device_ids=[local_rank], output_device=local_rank, find_unused_parameters=True)
 
     elif torch.cuda.device_count() > 1 and device.type == "cuda":
         model = WrappedDataParallel(model) if not isinstance(model, DataParallel) else WrappedDataParallel(model.module)
@@ -315,6 +345,8 @@ def _init_amp(model, device, optimizer=None, use_amp=None):
         if AMP_AVAILABLE:
             model, optimizer = amp.initialize(model, optimizer, opt_level=use_amp)
         else:
-            logger.warning(f"Can't find AMP although you specificed to use amp with level {use_amp}. Will continue without AMP ...")
+            logger.warning(
+                f"Can't find AMP although you specificed to use amp with level {use_amp}. Will continue without AMP ..."
+            )
 
     return model, optimizer
