@@ -1,6 +1,9 @@
+from haystack.utils.import_utils import safe_import
+
 from haystack.nodes.base import BaseComponent
+
+Crawler = safe_import("haystack.nodes.connector.crawler", "Crawler", "crawler")  # Has optional dependencies
 from haystack.nodes.answer_generator import BaseGenerator, RAGenerator, Seq2SeqGenerator
-from haystack.nodes.connector import Crawler
 from haystack.nodes.document_classifier import BaseDocumentClassifier, TransformersDocumentClassifier
 from haystack.nodes.evaluator import EvalDocuments, EvalAnswers
 from haystack.nodes.extractor import EntityExtractor, simplify_ner_for_qa
@@ -35,4 +38,4 @@ from haystack.nodes.retriever import (
     TableTextRetriever,
 )
 from haystack.nodes.summarizer import BaseSummarizer, TransformersSummarizer
-from haystack.nodes.translator import BaseTranslator, TransformersTranslator
\ No newline at end of file
+from haystack.nodes.translator import BaseTranslator, TransformersTranslator
