@@ -26,9 +26,9 @@ class BaseReader(BaseComponent):
         pass
 
     @staticmethod
-    def _calc_no_answer(no_ans_gaps: Sequence[float],
-                        best_score_answer: float,
-                        use_confidence_scores: bool = True) -> Tuple[Answer, float]:
+    def _calc_no_answer(
+        no_ans_gaps: Sequence[float], best_score_answer: float, use_confidence_scores: bool = True
+    ) -> Tuple[Answer, float]:
         # "no answer" scores and positive answers scores are difficult to compare, because
         # + a positive answer score is related to one specific document
         # - a "no answer" score is related to all input documents
@@ -40,18 +40,24 @@ class BaseReader(BaseComponent):
         max_no_ans_gap = np.max(no_ans_gap_array)
         # all passages "no answer" as top score
         if np.sum(no_ans_gap_array < 0) == len(no_ans_gap_array):
-            no_ans_score = best_score_answer - max_no_ans_gap  # max_no_ans_gap is negative, so it increases best pos score
+            no_ans_score = (
+                best_score_answer - max_no_ans_gap
+            )  # max_no_ans_gap is negative, so it increases best pos score
         else:  # case: at least one passage predicts an answer (positive no_ans_gap)
             no_ans_score = best_score_answer - max_no_ans_gap
 
-        no_ans_prediction = Answer(answer="",
-                                   type="extractive",
-                                   score=float(expit(np.asarray(no_ans_score) / 8)) if use_confidence_scores else no_ans_score,  # just a pseudo prob for now or old score,
-                                   context=None,
-                                   offsets_in_context=[Span(start=0, end=0)],
-                                   offsets_in_document=[Span(start=0, end=0)],
-                                   document_id=None,
-                                   meta=None)
+        no_ans_prediction = Answer(
+            answer="",
+            type="extractive",
+            score=float(expit(np.asarray(no_ans_score) / 8))
+            if use_confidence_scores
+            else no_ans_score,  # just a pseudo prob for now or old score,
+            context=None,
+            offsets_in_context=[Span(start=0, end=0)],
+            offsets_in_document=[Span(start=0, end=0)],
+            document_id=None,
+            meta=None,
+        )
 
         return no_ans_prediction, max_no_ans_gap
 
@@ -79,7 +85,9 @@ class BaseReader(BaseComponent):
             results = {"answers": []}
 
         # Add corresponding document_name and more meta data, if an answer contains the document_id
-        results["answers"] = [BaseReader.add_doc_meta_data_to_answer(documents=documents, answer=answer) for answer in results["answers"]]
+        results["answers"] = [
+            BaseReader.add_doc_meta_data_to_answer(documents=documents, answer=answer) for answer in results["answers"]
+        ]
 
         # run evaluation with labels as node inputs
         if add_isolated_node_eval and labels is not None:
@@ -87,12 +95,15 @@ class BaseReader(BaseComponent):
             results_label_input = predict(query=query, documents=relevant_documents, top_k=top_k)
 
             # Add corresponding document_name and more meta data, if an answer contains the document_id
-            results["answers_isolated"] = [BaseReader.add_doc_meta_data_to_answer(documents=documents, answer=answer) for answer in results_label_input["answers"]]
+            results["answers_isolated"] = [
+                BaseReader.add_doc_meta_data_to_answer(documents=documents, answer=answer)
+                for answer in results_label_input["answers"]
+            ]
 
         return results, "output_1"
 
     def run_batch(self, query_doc_list: List[Dict], top_k: Optional[int] = None):
-        """ A unoptimized implementation of running Reader queries in batch """
+        """A unoptimized implementation of running Reader queries in batch"""
         self.query_count += len(query_doc_list)
         results = []
         if query_doc_list:
@@ -107,7 +118,8 @@ class BaseReader(BaseComponent):
         return {"results": results}, "output_1"
 
     def timing(self, fn, attr_name):
-        """Wrapper method used to time functions. """
+        """Wrapper method used to time functions."""
+
         @wraps(fn)
         def wrapper(*args, **kwargs):
             if attr_name not in self.__dict__:
@@ -117,6 +129,7 @@ class BaseReader(BaseComponent):
             toc = perf_counter()
             self.__dict__[attr_name] += toc - tic
             return ret
+
         return wrapper
 
     def print_time(self):
@@ -128,5 +141,3 @@ class BaseReader(BaseComponent):
             print(f"Queries Performed: {self.query_count}")
             print(f"Query time: {self.query_time}s")
             print(f"{self.query_time / self.query_count} seconds per query")
-
-
