@@ -1,18 +1,15 @@
-from haystack.utils.preprocessing import (
-    convert_files_to_dicts, 
-    tika_convert_files_to_dicts
-)
+from haystack.utils.preprocessing import convert_files_to_dicts, tika_convert_files_to_dicts
 from haystack.utils.import_utils import fetch_archive_from_http
 from haystack.utils.cleaning import clean_wiki_text
 from haystack.utils.doc_store import (
     launch_es,
     launch_milvus,
-    launch_open_distro_es,
     launch_opensearch,
     launch_weaviate,
     stop_opensearch,
     stop_service,
 )
+from haystack.utils.deepsetcloud import DeepsetCloud, DeepsetCloudError
 from haystack.utils.export_utils import (
     print_answers,
     print_documents,
@@ -20,4 +17,4 @@ from haystack.utils.export_utils import (
     export_answers_to_csv,
     convert_labels_to_squad,
 )
-from haystack.utils.squad_data import SquadData
\ No newline at end of file
+from haystack.utils.squad_data import SquadData
