@@ -23,15 +23,14 @@ def print_answers(results: dict, details: str = "all", max_text_len: Optional[in
     :return: None
     """
     # Defines the fields to keep in the Answer for each detail level
-    fields_to_keep_by_level = {
-        "minimum": ["answer", "context"],
-        "medium": ["answer", "context", "score"]
-    }
+    fields_to_keep_by_level = {"minimum": ["answer", "context"], "medium": ["answer", "context", "score"]}
 
     if not "answers" in results.keys():
-        raise ValueError("The results object does not seem to come from a Reader: "
-                         f"it does not contain the 'answers' key, but only: {results.keys()}.  "
-                         "Try print_documents or print_questions.")
+        raise ValueError(
+            "The results object does not seem to come from a Reader: "
+            f"it does not contain the 'answers' key, but only: {results.keys()}.  "
+            "Try print_documents or print_questions."
+        )
 
     if "query" in results.keys():
         print(f"\nQuery: {results['query']}\nAnswers:")
@@ -43,9 +42,13 @@ def print_answers(results: dict, details: str = "all", max_text_len: Optional[in
     filtered_answers = []
     if details in fields_to_keep_by_level.keys():
         for ans in answers:
-            filtered_ans = {field: getattr(ans, field) for field in fields_to_keep_by_level[details] if getattr(ans, field) is not None}
+            filtered_ans = {
+                field: getattr(ans, field)
+                for field in fields_to_keep_by_level[details]
+                if getattr(ans, field) is not None
+            }
             filtered_answers.append(filtered_ans)
-    elif details == "all":  
+    elif details == "all":
         filtered_answers = answers
     else:
         valid_values = ", ".join(fields_to_keep_by_level.keys()) + " and 'all'"
@@ -59,10 +62,12 @@ def print_answers(results: dict, details: str = "all", max_text_len: Optional[in
             if getattr(ans, "context") and len(ans.context) > max_text_len:
                 ans.context = ans.context[:max_text_len] + "..."
 
-    pp.pprint(filtered_answers) 
+    pp.pprint(filtered_answers)
 
 
-def print_documents(results: dict, max_text_len: Optional[int] = None, print_name: bool = True, print_meta: bool = False):
+def print_documents(
+    results: dict, max_text_len: Optional[int] = None, print_name: bool = True, print_meta: bool = False
+):
     """
     Utility that prints a compressed representation of the documents returned by a pipeline.
     :param max_text_lenght: shorten the document's content to a maximum number of chars. if None, does not cut.
@@ -71,13 +76,15 @@ def print_documents(results: dict, max_text_len: Optional[int] = None, print_nam
     """
     print(f"\nQuery: {results['query']}\n")
     pp = pprint.PrettyPrinter(indent=4)
-    
+
     # Verify that the input contains Documents under the `document` key
     if any(not isinstance(doc, Document) for doc in results["documents"]):
-        raise ValueError("This results object does not contain `Document` objects under the `documents` key. "
-                         "Please make sure the last node of your pipeline makes proper use of the "
-                         "new Haystack primitive objects, and if you're using Haystack nodes/pipelines only, "
-                         "please report this as a bug.")
+        raise ValueError(
+            "This results object does not contain `Document` objects under the `documents` key. "
+            "Please make sure the last node of your pipeline makes proper use of the "
+            "new Haystack primitive objects, and if you're using Haystack nodes/pipelines only, "
+            "please report this as a bug."
+        )
 
     for doc in results["documents"]:
         content = doc.content
@@ -96,7 +103,7 @@ def print_questions(results: dict):
     """
     Utility to print the output of a question generating pipeline in a readable format.
     """
-    if "generated_questions" in results.keys():        
+    if "generated_questions" in results.keys():
         print("\nGenerated questions:")
         for result in results["generated_questions"]:
             for question in result["questions"]:
@@ -110,18 +117,22 @@ def print_questions(results: dict):
 
                 # Verify that the pairs contains Answers under the `answer` key
                 if not isinstance(answer, Answer):
-                    raise ValueError("This results object does not contain `Answer` objects under the `answers` "
-                                    "key of the generated question/answer pairs. "
-                                    "Please make sure the last node of your pipeline makes proper use of the "
-                                    "new Haystack primitive objects, and if you're using Haystack nodes/pipelines only, "
-                                    "please report this as a bug.")
+                    raise ValueError(
+                        "This results object does not contain `Answer` objects under the `answers` "
+                        "key of the generated question/answer pairs. "
+                        "Please make sure the last node of your pipeline makes proper use of the "
+                        "new Haystack primitive objects, and if you're using Haystack nodes/pipelines only, "
+                        "please report this as a bug."
+                    )
                 print(f"      A: {answer.answer}")
 
     else:
-        raise ValueError("This object does not seem to be the output " 
-              "of a question generating pipeline: does not contain neither "
-              f"'generated_questions' nor 'results', but only: {results.keys()}. "
-              " Try `print_answers` or `print_documents`.")
+        raise ValueError(
+            "This object does not seem to be the output "
+            "of a question generating pipeline: does not contain neither "
+            f"'generated_questions' nor 'results', but only: {results.keys()}. "
+            " Try `print_answers` or `print_documents`."
+        )
 
 
 def export_answers_to_csv(agg_results: list, output_file):
@@ -137,7 +148,7 @@ def export_answers_to_csv(agg_results: list, output_file):
     assert "query" in agg_results[0], f"Wrong format used for {agg_results[0]}"
     assert "answers" in agg_results[0], f"Wrong format used for {agg_results[0]}"
 
-    data = {} # type: Dict[str, List[Any]]
+    data = {}  # type: Dict[str, List[Any]]
     data["query"] = []
     data["prediction"] = []
     data["prediction_rank"] = []
@@ -162,7 +173,7 @@ def convert_labels_to_squad(labels_file: str):
     :param labels_file: path for export file from the labeling tool
     :return:
     """
-    with open(labels_file, encoding='utf-8') as label_file:
+    with open(labels_file, encoding="utf-8") as label_file:
         labels = json.load(label_file)
 
     labels_grouped_by_documents = defaultdict(list)
@@ -175,10 +186,7 @@ def convert_labels_to_squad(labels_file: str):
         for label in labels:
             doc = DocumentORM.query.get(label["document_id"])
 
-            assert (
-                    doc.content[label["start_offset"]: label["end_offset"]]
-                == label["selected_text"]
-            )
+            assert doc.content[label["start_offset"] : label["end_offset"]] == label["selected_text"]
 
             qas.append(
                 {
@@ -196,13 +204,9 @@ def convert_labels_to_squad(labels_file: str):
                 }
             )
 
-        squad_format_label = {
-            "paragraphs": [
-                {"qas": qas, "context": doc.content, "document_id": document_id}
-            ]
-        }
+        squad_format_label = {"paragraphs": [{"qas": qas, "context": doc.content, "document_id": document_id}]}
 
         labels_in_squad_format["data"].append(squad_format_label)
 
-    with open("labels_in_squad_format.json", "w+", encoding='utf-8') as outfile:
+    with open("labels_in_squad_format.json", "w+", encoding="utf-8") as outfile:
         json.dump(labels_in_squad_format, outfile)
