@@ -1,10 +1,11 @@
+from typing import Optional, List, Union
+
 import json
 import logging
 import os
 import shutil
 import uuid
 from pathlib import Path
-from typing import Optional, List
 
 from fastapi import APIRouter, UploadFile, File, Form, HTTPException, Depends
 from pydantic import BaseModel
@@ -13,23 +14,33 @@ from haystack.pipelines.base import Pipeline
 from rest_api.config import PIPELINE_YAML_PATH, FILE_UPLOAD_PATH, INDEXING_PIPELINE_NAME
 from rest_api.controller.utils import as_form
 
+
 logger = logging.getLogger(__name__)
 router = APIRouter()
 
 try:
-    _, pipeline_config, definitions = Pipeline._read_yaml(
-        path=Path(PIPELINE_YAML_PATH), pipeline_name=INDEXING_PIPELINE_NAME, overwrite_with_env_variables=True
+    pipeline_config = Pipeline._read_pipeline_config_from_yaml(Path(PIPELINE_YAML_PATH))
+    pipeline_definition = Pipeline._get_pipeline_definition(
+        pipeline_config=pipeline_config, pipeline_name=INDEXING_PIPELINE_NAME
+    )
+    definitions = Pipeline._get_component_definitions(
+        pipeline_config=pipeline_config, overwrite_with_env_variables=True
     )
     # Since each instance of FAISSDocumentStore creates an in-memory FAISS index, the Indexing & Query Pipelines would
-    # end up with different indices. The same applies for InMemoryDocumentStore. The check below prevents creation of 
-    # Indexing Pipelines with FAISSDocumentStore or InMemoryDocumentStore.   
+    # end up with different indices. The same applies for InMemoryDocumentStore. The check below prevents creation of
+    # Indexing Pipelines with FAISSDocumentStore or InMemoryDocumentStore.
     is_faiss_or_inmemory_present = False
-    for node in pipeline_config["nodes"]:
-        if definitions[node["name"]]["type"] == "FAISSDocumentStore" or definitions[node["name"]]["type"] == "InMemoryDocumentStore":
+    for node in pipeline_definition["nodes"]:
+        if (
+            definitions[node["name"]]["type"] == "FAISSDocumentStore"
+            or definitions[node["name"]]["type"] == "InMemoryDocumentStore"
+        ):
             is_faiss_or_inmemory_present = True
             break
     if is_faiss_or_inmemory_present:
-        logger.warning("Indexing Pipeline with FAISSDocumentStore or InMemoryDocumentStore is not supported with the REST APIs.")
+        logger.warning(
+            "Indexing Pipeline with FAISSDocumentStore or InMemoryDocumentStore is not supported with the REST APIs."
+        )
         INDEXING_PIPELINE = None
     else:
         INDEXING_PIPELINE = Pipeline.load_from_yaml(Path(PIPELINE_YAML_PATH), pipeline_name=INDEXING_PIPELINE_NAME)
@@ -38,11 +49,12 @@ except KeyError:
     logger.warning("Indexing Pipeline not found in the YAML configuration. File Upload API will not be available.")
 
 
-os.makedirs(FILE_UPLOAD_PATH, exist_ok=True)  # create directory for uploading files
+# create directory for uploading files
+os.makedirs(FILE_UPLOAD_PATH, exist_ok=True)
 
 
 @as_form
-class FileConverterParams(BaseModel):    
+class FileConverterParams(BaseModel):
     remove_numeric_tables: Optional[bool] = None
     valid_languages: Optional[List[str]] = None
 
@@ -65,16 +77,24 @@ class Response(BaseModel):
 @router.post("/file-upload")
 def upload_file(
     files: List[UploadFile] = File(...),
-    meta: Optional[str] = Form("null"),  # JSON serialized string
-    fileconverter_params: FileConverterParams = Depends(FileConverterParams.as_form),
-    preprocessor_params: PreprocessorParams = Depends(PreprocessorParams.as_form)
+    # JSON serialized string
+    meta: Optional[str] = Form("null"),  # type: ignore
+    fileconverter_params: FileConverterParams = Depends(FileConverterParams.as_form),  # type: ignore
+    preprocessor_params: PreprocessorParams = Depends(PreprocessorParams.as_form),  # type: ignore
 ):
+    """
+    You can use this endpoint to upload a file for indexing
+    (see https://haystack.deepset.ai/guides/rest-api#indexing-documents-in-the-haystack-rest-api-document-store).
+    """
     if not INDEXING_PIPELINE:
         raise HTTPException(status_code=501, detail="Indexing Pipeline is not configured.")
 
     file_paths: list = []
     file_metas: list = []
-    meta = json.loads(meta) or {}
+
+    meta_form = json.loads(meta) or {}  # type: ignore
+    if not isinstance(meta_form, dict):
+        raise HTTPException(status_code=500, detail=f"The meta field must be a dict or None, not {type(meta_form)}")
 
     for file in files:
         try:
@@ -83,17 +103,17 @@ def upload_file(
                 shutil.copyfileobj(file.file, buffer)
 
             file_paths.append(file_path)
-            meta["name"] = file.filename
-            file_metas.append(meta)
+            meta_form["name"] = file.filename
+            file_metas.append(meta_form)
         finally:
             file.file.close()
 
     INDEXING_PIPELINE.run(
-            file_paths=file_paths,
-            meta=file_metas,
-            params={
-                "TextFileConverter": fileconverter_params.dict(), 
-                "PDFFileConverter": fileconverter_params.dict(),
-                "Preprocessor": preprocessor_params.dict()
-            },
+        file_paths=file_paths,
+        meta=file_metas,
+        params={
+            "TextFileConverter": fileconverter_params.dict(),
+            "PDFFileConverter": fileconverter_params.dict(),
+            "Preprocessor": preprocessor_params.dict(),
+        },
     )
