@@ -3,13 +3,17 @@ from pathlib import Path
 import pytest
 import ray
 
-from haystack.pipeline import RayPipeline
+from haystack.pipelines import RayPipeline
+
+from conftest import SAMPLES_PATH
 
 
 @pytest.mark.parametrize("document_store_with_docs", ["elasticsearch"], indirect=True)
 def test_load_pipeline(document_store_with_docs):
     pipeline = RayPipeline.load_from_yaml(
-        Path("samples/pipeline/test_pipeline.yaml"), pipeline_name="ray_query_pipeline", num_cpus=8,
+        SAMPLES_PATH / "pipeline" / "test_pipeline.yaml",
+        pipeline_name="ray_query_pipeline",
+        num_cpus=8,
     )
     prediction = pipeline.run(query="Who lives in Berlin?", params={"Retriever": {"top_k": 10}, "Reader": {"top_k": 3}})
 
@@ -24,6 +28,7 @@ def shutdown_ray():
     yield
     try:
         import ray
+
         ray.shutdown()
     except:
         pass
