@@ -1,4 +1,5 @@
 from pathlib import Path
+from collections import defaultdict
 
 import os
 import math
@@ -12,9 +13,17 @@ from haystack.pipelines import (
     RootNode,
     MostSimilarDocumentsPipeline,
 )
-from haystack.nodes import DensePassageRetriever,  ElasticsearchRetriever, SklearnQueryClassifier, TransformersQueryClassifier, JoinDocuments
+from haystack.nodes import (
+    DensePassageRetriever,
+    ElasticsearchRetriever,
+    SklearnQueryClassifier,
+    TransformersQueryClassifier,
+    JoinDocuments,
+)
 from haystack.schema import Document
 
+from conftest import SAMPLES_PATH
+
 
 @pytest.mark.parametrize(
     "retriever,document_store",
@@ -61,7 +70,9 @@ def test_faq_pipeline(retriever, document_store):
     assert output["answers"][0].answer.startswith("Using tests")
 
     if isinstance(document_store, ElasticsearchDocumentStore):
-        output = pipeline.run(query="How to test this?", params={"Retriever": {"filters": {"source": ["wiki2"]}, "top_k": 5}})
+        output = pipeline.run(
+            query="How to test this?", params={"Retriever": {"filters": {"source": ["wiki2"]}, "top_k": 5}}
+        )
         assert len(output["answers"]) == 1
 
 
@@ -88,13 +99,13 @@ def test_document_search_pipeline(retriever, document_store):
 
 
 @pytest.mark.parametrize(
-        "retriever,document_store",
-        [
-            ("embedding", "faiss"),
-            ("embedding", "milvus"),
-            ("embedding", "elasticsearch"),
-        ],
-        indirect=True,
+    "retriever,document_store",
+    [
+        ("embedding", "faiss"),
+        ("embedding", "milvus"),
+        ("embedding", "elasticsearch"),
+    ],
+    indirect=True,
 )
 def test_most_similar_documents_pipeline(retriever, document_store):
     documents = [
@@ -126,8 +137,7 @@ def test_most_similar_documents_pipeline(retriever, document_store):
 
 @pytest.mark.elasticsearch
 @pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
-@pytest.mark.parametrize("reader", ["farm"], indirect=True)
-def test_join_document_pipeline(document_store_dot_product_with_docs, reader):
+def test_join_merge_no_weights(document_store_dot_product_with_docs):
     es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
     dpr = DensePassageRetriever(
         document_store=document_store_dot_product_with_docs,
@@ -139,35 +149,102 @@ def test_join_document_pipeline(document_store_dot_product_with_docs, reader):
 
     query = "Where does Carla live?"
 
-    # test merge without weights
     join_node = JoinDocuments(join_mode="merge")
     p = Pipeline()
     p.add_node(component=es, name="R1", inputs=["Query"])
     p.add_node(component=dpr, name="R2", inputs=["Query"])
     p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
     results = p.run(query=query)
-    assert len(results["documents"]) == 3
+    assert len(results["documents"]) == 5
+
+
+@pytest.mark.elasticsearch
+@pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
+def test_join_merge_with_weights(document_store_dot_product_with_docs):
+    es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
+    dpr = DensePassageRetriever(
+        document_store=document_store_dot_product_with_docs,
+        query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+        passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base",
+        use_gpu=False,
+    )
+    document_store_dot_product_with_docs.update_embeddings(dpr)
+
+    query = "Where does Carla live?"
 
-    # test merge with weights
     join_node = JoinDocuments(join_mode="merge", weights=[1000, 1], top_k_join=2)
     p = Pipeline()
     p.add_node(component=es, name="R1", inputs=["Query"])
     p.add_node(component=dpr, name="R2", inputs=["Query"])
     p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
     results = p.run(query=query)
-    assert math.isclose(results["documents"][0].score, 0.5350644373470798, rel_tol=0.0001)
+    assert math.isclose(results["documents"][0].score, 0.5481393431183286, rel_tol=0.0001)
     assert len(results["documents"]) == 2
 
-    # test concatenate
+
+@pytest.mark.elasticsearch
+@pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
+def test_join_concatenate(document_store_dot_product_with_docs):
+    es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
+    dpr = DensePassageRetriever(
+        document_store=document_store_dot_product_with_docs,
+        query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+        passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base",
+        use_gpu=False,
+    )
+    document_store_dot_product_with_docs.update_embeddings(dpr)
+
+    query = "Where does Carla live?"
+
     join_node = JoinDocuments(join_mode="concatenate")
     p = Pipeline()
     p.add_node(component=es, name="R1", inputs=["Query"])
     p.add_node(component=dpr, name="R2", inputs=["Query"])
     p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
     results = p.run(query=query)
-    assert len(results["documents"]) == 3
+    assert len(results["documents"]) == 5
+
+
+@pytest.mark.elasticsearch
+@pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
+def test_join_concatenate_with_topk(document_store_dot_product_with_docs):
+    es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
+    dpr = DensePassageRetriever(
+        document_store=document_store_dot_product_with_docs,
+        query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+        passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base",
+        use_gpu=False,
+    )
+    document_store_dot_product_with_docs.update_embeddings(dpr)
+
+    query = "Where does Carla live?"
+
+    join_node = JoinDocuments(join_mode="concatenate")
+    p = Pipeline()
+    p.add_node(component=es, name="R1", inputs=["Query"])
+    p.add_node(component=dpr, name="R2", inputs=["Query"])
+    p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
+    one_result = p.run(query=query, params={"Join": {"top_k_join": 1}})
+    two_results = p.run(query=query, params={"Join": {"top_k_join": 2}})
+    assert len(one_result["documents"]) == 1
+    assert len(two_results["documents"]) == 2
+
+
+@pytest.mark.elasticsearch
+@pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
+@pytest.mark.parametrize("reader", ["farm"], indirect=True)
+def test_join_with_reader(document_store_dot_product_with_docs, reader):
+    es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
+    dpr = DensePassageRetriever(
+        document_store=document_store_dot_product_with_docs,
+        query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+        passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base",
+        use_gpu=False,
+    )
+    document_store_dot_product_with_docs.update_embeddings(dpr)
+
+    query = "Where does Carla live?"
 
-    # test join_node with reader
     join_node = JoinDocuments()
     p = Pipeline()
     p.add_node(component=es, name="R1", inputs=["Query"])
@@ -175,10 +252,43 @@ def test_join_document_pipeline(document_store_dot_product_with_docs, reader):
     p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
     p.add_node(component=reader, name="Reader", inputs=["Join"])
     results = p.run(query=query)
-    #check whether correct answer is within top 2 predictions
+    # check whether correct answer is within top 2 predictions
     assert results["answers"][0].answer == "Berlin" or results["answers"][1].answer == "Berlin"
 
 
+@pytest.mark.elasticsearch
+@pytest.mark.parametrize("document_store_dot_product_with_docs", ["elasticsearch"], indirect=True)
+def test_join_with_rrf(document_store_dot_product_with_docs):
+    es = ElasticsearchRetriever(document_store=document_store_dot_product_with_docs)
+    dpr = DensePassageRetriever(
+        document_store=document_store_dot_product_with_docs,
+        query_embedding_model="facebook/dpr-question_encoder-single-nq-base",
+        passage_embedding_model="facebook/dpr-ctx_encoder-single-nq-base",
+        use_gpu=False,
+    )
+    document_store_dot_product_with_docs.update_embeddings(dpr)
+
+    query = "Where does Carla live?"
+
+    join_node = JoinDocuments(join_mode="reciprocal_rank_fusion")
+    p = Pipeline()
+    p.add_node(component=es, name="R1", inputs=["Query"])
+    p.add_node(component=dpr, name="R2", inputs=["Query"])
+    p.add_node(component=join_node, name="Join", inputs=["R1", "R2"])
+    results = p.run(query=query)
+
+    # list of precalculated expected results
+    expected_scores = [
+        0.03278688524590164,
+        0.03200204813108039,
+        0.03200204813108039,
+        0.031009615384615385,
+        0.031009615384615385,
+    ]
+
+    assert all([doc.score == expected_scores[idx] for idx, doc in enumerate(results["documents"])])
+
+
 def test_query_keyword_statement_classifier():
     class KeywordOutput(RootNode):
         outgoing_edges = 2
@@ -244,15 +354,11 @@ def test_query_keyword_statement_classifier():
 def test_indexing_pipeline_with_classifier(document_store):
     # test correct load of indexing pipeline from yaml
     pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline.yaml", pipeline_name="indexing_pipeline_with_classifier"
-    )
-    pipeline.run(
-        file_paths=Path(__file__).parent/"samples"/"pdf"/"sample_pdf_1.pdf"
+        SAMPLES_PATH / "pipeline" / "test_pipeline.yaml", pipeline_name="indexing_pipeline_with_classifier"
     )
+    pipeline.run(file_paths=SAMPLES_PATH / "pdf" / "sample_pdf_1.pdf")
     # test correct load of query pipeline from yaml
-    pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline.yaml", pipeline_name="query_pipeline"
-    )
+    pipeline = Pipeline.load_from_yaml(SAMPLES_PATH / "pipeline" / "test_pipeline.yaml", pipeline_name="query_pipeline")
     prediction = pipeline.run(
         query="Who made the PDF specification?", params={"ESRetriever": {"top_k": 10}, "Reader": {"top_k": 3}}
     )
@@ -267,14 +373,12 @@ def test_indexing_pipeline_with_classifier(document_store):
 def test_query_pipeline_with_document_classifier(document_store):
     # test correct load of indexing pipeline from yaml
     pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline.yaml", pipeline_name="indexing_pipeline"
-    )
-    pipeline.run(
-        file_paths=Path(__file__).parent/"samples"/"pdf"/"sample_pdf_1.pdf"
+        SAMPLES_PATH / "pipeline" / "test_pipeline.yaml", pipeline_name="indexing_pipeline"
     )
+    pipeline.run(file_paths=SAMPLES_PATH / "pdf" / "sample_pdf_1.pdf")
     # test correct load of query pipeline from yaml
     pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline.yaml", pipeline_name="query_pipeline_with_document_classifier"
+        SAMPLES_PATH / "pipeline" / "test_pipeline.yaml", pipeline_name="query_pipeline_with_document_classifier"
     )
     prediction = pipeline.run(
         query="Who made the PDF specification?", params={"ESRetriever": {"top_k": 10}, "Reader": {"top_k": 3}}
@@ -289,28 +393,23 @@ def test_existing_faiss_document_store():
     clean_faiss_document_store()
 
     pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline_faiss_indexing.yaml", pipeline_name="indexing_pipeline"
-    )
-    pipeline.run(
-        file_paths=Path(__file__).parent/"samples"/"pdf"/"sample_pdf_1.pdf"
+        SAMPLES_PATH / "pipeline" / "test_pipeline_faiss_indexing.yaml", pipeline_name="indexing_pipeline"
     )
+    pipeline.run(file_paths=SAMPLES_PATH / "pdf" / "sample_pdf_1.pdf")
 
     new_document_store = pipeline.get_document_store()
-    new_document_store.save('existing_faiss_document_store')
+    new_document_store.save("existing_faiss_document_store")
 
     # test correct load of query pipeline from yaml
     pipeline = Pipeline.load_from_yaml(
-        Path(__file__).parent/"samples"/"pipeline"/"test_pipeline_faiss_retrieval.yaml", pipeline_name="query_pipeline"
+        SAMPLES_PATH / "pipeline" / "test_pipeline_faiss_retrieval.yaml", pipeline_name="query_pipeline"
     )
 
-    retriever = pipeline.get_node("DPRRetriever")
-    existing_document_store = retriever.document_store
-    faiss_index = existing_document_store.faiss_indexes['document']
+    existing_document_store = pipeline.get_document_store()
+    faiss_index = existing_document_store.faiss_indexes["document"]
     assert faiss_index.ntotal == 2
 
-    prediction = pipeline.run(
-        query="Who made the PDF specification?", params={"DPRRetriever": {"top_k": 10}}
-    )
+    prediction = pipeline.run(query="Who made the PDF specification?", params={"DPRRetriever": {"top_k": 10}})
 
     assert prediction["query"] == "Who made the PDF specification?"
     assert len(prediction["documents"]) == 2
@@ -318,9 +417,9 @@ def test_existing_faiss_document_store():
 
 
 def clean_faiss_document_store():
-    if Path('existing_faiss_document_store').exists():
-        os.remove('existing_faiss_document_store')
-    if Path('existing_faiss_document_store.json').exists():
-        os.remove('existing_faiss_document_store.json')
-    if Path('faiss_document_store.db').exists():
-        os.remove('faiss_document_store.db')
+    if Path("existing_faiss_document_store").exists():
+        os.remove("existing_faiss_document_store")
+    if Path("existing_faiss_document_store.json").exists():
+        os.remove("existing_faiss_document_store.json")
+    if Path("faiss_document_store.db").exists():
+        os.remove("faiss_document_store.db")
